﻿; *****************************************************************************
; DynLib.pb
; by Luis
; http://luis.no-ip.net
; *****************************************************************************

XIncludeFile "DynLib.pbi"

Module DynLib

#DL_File = 0
#DL_SysTemp = 1
#DL_Memory = 2

Structure T_REGISTERED_LIBRARY
 reg.i
 handle.i
 source.i
 file$
 fileTemp$
 *start
 *end
EndStructure
 
Structure T_THIS
 Counter.i
 SysTempDir$
 BaseDirectory$
 List RegisteredLibrariesList.T_REGISTERED_LIBRARY()
EndStructure : Global this.T_THIS

this\SysTempDir$ = GetTemporaryDirectory()

CompilerIf Defined(PS$, #PB_Constant) = 0
 CompilerIf (#PB_Compiler_OS = #PB_OS_Windows)
  #PS$ = "\"
 CompilerElse
  #PS$ = "/"
 CompilerEndIf
CompilerEndIf

Procedure.s GenerateDirectoryPath() 
 ProcedureReturn this\SysTempDir$ + "dynlib_" + Str(Random(9999999,1000000)) ; ie: c:\temp\dynlib_4178203
EndProcedure   

Procedure.s CreateBaseDirectory()
 Protected ret, try_again = 5
 Protected dir$
 
 While try_again ; tries with different directories in case of collision        
    dir$ = GenerateDirectoryPath()    
    ret = FileSize(dir$)
    If (ret <> -2) Or (ret = -1) ; if not a dir and not a file
        If CreateDirectory(dir$)
            ProcedureReturn dir$
        EndIf
    EndIf
    try_again - 1
    Delay(10)
 Wend
 
 ProcedureReturn "" 
EndProcedure

Procedure.i IsDirEmpty (dir$)
 Protected hd, Count
 
 hd = ExamineDirectory(#PB_Any, dir$, "*.*")
 
 While NextDirectoryEntry(hd)
    If DirectoryEntryName(hd) = "."
        Continue
    EndIf
    If DirectoryEntryName(hd) = ".."
        Continue
    EndIf
    Count + 1
 Wend
 FinishDirectory(hd)

 ProcedureReturn Bool(Count = 0)
EndProcedure

Procedure.s ExtractToTemp (file$, *start, *end)
 Protected hfo
 Protected FullPath$
 
 If this\BaseDirectory$ <> ""
    If FileSize(this\BaseDirectory$) <> -2 ; check if the directory it's there
        this\BaseDirectory$ = "" ; forget about it, it's gone and we need a new one
    EndIf
 EndIf
 
 If this\BaseDirectory$ = ""
    this\BaseDirectory$ = CreateBaseDirectory()    
    If this\BaseDirectory$ = ""
        ProcedureReturn ""
    EndIf
 EndIf
 
 FullPath$ = this\BaseDirectory$ + #PS$ + file$ 
 
 hfo = CreateFile(#PB_Any, FullPath$)
 
 If hfo
    WriteData(hfo, *start, *end - *start)
    CloseFile(hfo)        
    ProcedureReturn FullPath$
 EndIf
 
 ProcedureReturn "" 
EndProcedure

Procedure.i FindByReg (reg)
 If reg
     ForEach this\RegisteredLibrariesList()
        If this\RegisteredLibrariesList()\reg = reg
            ProcedureReturn 1
        EndIf
     Next 
 EndIf
 ProcedureReturn 0
EndProcedure

Procedure.i FindByHandle (handle)
 If handle 
     ForEach this\RegisteredLibrariesList()
        If this\RegisteredLibrariesList()\handle = handle
            ProcedureReturn 1
        EndIf
     Next 
 EndIf
 ProcedureReturn 0
EndProcedure

;-[ PUBLIC ]

Procedure.i RegisterFile (file$)
 If FileSize(file$) = -1
    ProcedureReturn 0
 EndIf
            
 If AddElement(this\RegisteredLibrariesList())
    this\Counter + 1
    this\RegisteredLibrariesList()\reg = this\Counter
    this\RegisteredLibrariesList()\source = #DL_File
    this\RegisteredLibrariesList()\file$ = file$
    this\RegisteredLibrariesList()\fileTemp$ = ""
    this\RegisteredLibrariesList()\handle = 0            
    this\RegisteredLibrariesList()\start = 0
    this\RegisteredLibrariesList()\end = 0
    ProcedureReturn this\Counter
 EndIf    

 ProcedureReturn 0
EndProcedure

Procedure.i RegisterSysTemp (file$, *start, *end)      
 If *start <= 0
    ProcedureReturn 0
 EndIf
 
 If *end <= 0
    ProcedureReturn 0
 EndIf        
 
 If FindString(file$, "\")
    ProcedureReturn 0 
 EndIf
 
 If FindString(file$, "/")
    ProcedureReturn 0
 EndIf
 
 If FindString(file$, ":")            
    ProcedureReturn 0
 EndIf
 
 If FindString(file$, "..")
    ProcedureReturn 0
 EndIf
 
 If this\SysTempDir$ = "" ; no temp dir ?
    CallDebugger ; this should never happen
    ProcedureReturn 0
 EndIf
        
 If AddElement(this\RegisteredLibrariesList())
    this\Counter + 1
    this\RegisteredLibrariesList()\reg = this\Counter
    this\RegisteredLibrariesList()\source = #DL_SysTemp
    this\RegisteredLibrariesList()\file$ = file$
    this\RegisteredLibrariesList()\fileTemp$ = ""
    this\RegisteredLibrariesList()\handle = 0            
    this\RegisteredLibrariesList()\start = *start
    this\RegisteredLibrariesList()\end= *end
    ProcedureReturn this\Counter    
 EndIf

 ProcedureReturn 0
EndProcedure

Procedure.i RegisterMemory (*start, *end)
 CompilerIf (#PB_Compiler_OS <> #PB_OS_Windows)   
    ProcedureReturn 0
 CompilerEndIf    
    
 If *start <= 0
    ProcedureReturn 0
 EndIf
 
 If *end <= 0
    ProcedureReturn 0
 EndIf        
       
 If AddElement(this\RegisteredLibrariesList())
    this\Counter + 1
    this\RegisteredLibrariesList()\reg = this\Counter
    this\RegisteredLibrariesList()\source = #DL_Memory
    this\RegisteredLibrariesList()\file$ = ""
    this\RegisteredLibrariesList()\fileTemp$ = ""
    this\RegisteredLibrariesList()\handle = 0            
    this\RegisteredLibrariesList()\start = *start
    this\RegisteredLibrariesList()\end= *end
    ProcedureReturn this\Counter
 EndIf 
 
 ProcedureReturn 0
EndProcedure

Procedure.i LoadLibrary (reg)
 Protected  hl, TempFile$
 
 If FindByReg(reg)
    If this\RegisteredLibrariesList()\handle = 0
        Select this\RegisteredLibrariesList()\source
            Case #DL_File
                hl = OpenLibrary(#PB_Any, this\RegisteredLibrariesList()\file$)
                If hl
                    this\RegisteredLibrariesList()\handle = hl
                EndIf
            Case #DL_SysTemp
                TempFile$ = ExtractToTemp(this\RegisteredLibrariesList()\file$, this\RegisteredLibrariesList()\start, this\RegisteredLibrariesList()\end)
                
                If TempFile$
                    hl = OpenLibrary(#PB_Any, TempFile$)
                    If hl
                        this\RegisteredLibrariesList()\handle = hl
                        this\RegisteredLibrariesList()\fileTemp$ = TempFile$                    
                    EndIf
                EndIf
            Case #DL_Memory
               CompilerIf (#PB_Compiler_OS = #PB_OS_Windows)
                hl = MemDll::LoadLibrary(this\RegisteredLibrariesList()\start)
                If hl
                    this\RegisteredLibrariesList()\handle = hl            
                EndIf            
               CompilerEndIf        
        EndSelect
    EndIf
 EndIf 
 
 ProcedureReturn hl
EndProcedure

Procedure.i GetProcAddress (handle, func$)
 Protected *fp
 
 If FindByHandle(handle)
    Select this\RegisteredLibrariesList()\source
        Case #DL_File
            *fp = GetFunction(handle, func$)
        Case #DL_SysTemp
            *fp = GetFunction(handle, func$)
        Case #DL_Memory
           CompilerIf (#PB_Compiler_OS = #PB_OS_Windows)
            *fp = MemDll::GetProcAddress(handle, func$)            
           CompilerEndIf
    EndSelect   
 EndIf
 
 ProcedureReturn *fp
EndProcedure

Procedure FreeSelectedLibrary()
 Protected TmpDir$
 Protected handle = this\RegisteredLibrariesList()\handle
 
 If handle = 0
    ProcedureReturn 
 EndIf
 
 Select this\RegisteredLibrariesList()\source
    Case #DL_File
        CloseLibrary(handle)
    Case #DL_SysTemp
        TmpDir$ = GetPathPart(this\RegisteredLibrariesList()\fileTemp$)
        CloseLibrary(handle)
        DeleteFile(this\RegisteredLibrariesList()\fileTemp$)
        If IsDirEmpty(TmpDir$)
            DeleteDirectory(TmpDir$, "")
        EndIf
    Case #DL_Memory
       CompilerIf (#PB_Compiler_OS = #PB_OS_Windows)
        MemDll::FreeLibrary(handle)
       CompilerEndIf
 EndSelect
 
 this\RegisteredLibrariesList()\handle = 0
EndProcedure

Procedure FreeLibrary (handle) 
 ; this closes a specific library 
 ; it does not invalidate the registration library handle
 If FindByHandle(handle)
    FreeSelectedLibrary()    
 EndIf
EndProcedure 

Procedure FreeLibraries() 
 ; this closes every library
 ForEach this\RegisteredLibrariesList() 
    FreeSelectedLibrary()    
 Next 
EndProcedure 

EndModule
; IDE Options = PureBasic 5.60 (Windows - x86)
; Folding = ----
; EnableXP
; EnableUser
; CPU = 1
; CompileSourceDirectory
; EnablePurifier
; EnableExeConstant
; EnableUnicode