﻿; *****************************************************************************
; DynLib.pbi
; by Luis
; http://luis.no-ip.net
;
; Dynamic library loader from file, extracted memory, direct memory.
; OS: Windows
; Build: x86, x64
; Requires: MemDll on Windows
;
; 1.0                                                 July 5, 2017      PB 5.60
; First release
; *****************************************************************************

; Module to load a dynamic library from:
; - a library file as done with the OpenLibrary() command.
; - an embedded copy in the exe after its extraction to the system temp dir.
; - an embedded copy in the exe directly from memory.
; 
; The loading from direct memory is supported only on Windows.
; If the library is extracted to the system temp dir, it is deleted when closed with FreeLibrary().
; With Register*() you get an intermediate handle to the specified lib.
;
; Later you use that handle with LoadLibrary() to get the actual handle from the OS.
; FreeLibrary() free (close) a specific library.
; FreeLibraries() free (close) all the open libraries.
; The reference number created with Register*() it's still valid, this means you can load (open)
; the same library again using LoadLibrary().
;
; RegisterFile (file$)
;   It's the usual loading from file, so the param must be a valid lib name 
;   with a valid path exactly as you do with OpenLibrary().
; RegisterSysTemp (file$, *start, *end)
;   The library is extracted from memory and written to the system temp dir, 
;   then opened from there exactly as you would do with OpenLibrary().
;   The first param is the file name which will be used for the temporary file, without any path.
;   The Module manages automatically the creation of a unique path in the system dir to avoid name collisions
;   with multiple instances of the same program or different programs using the same DLLs.
; RegisterMemory (*start, *end)
;   The library is loaded directly from memory with a custom loader (MemDll).
;   See MemDll.pbi for more information.
;
; Only the registering part it's specific to the type of source, the rest of the code is the same for file, systemp or memory.
; This is handy to switch from one type to another by just changing the Register*() call.
 
CompilerIf (#PB_Compiler_OS = #PB_OS_Windows)
XIncludeFile "../MemDll/MemDll.pb"
CompilerEndIf

DeclareModule DynLib
EnableExplicit

Declare.i 	RegisterFile (file$)
Declare.i 	RegisterSysTemp (file$, *start, *end)
Declare.i 	RegisterMemory (*start, *end)
Declare.i 	LoadLibrary (reg)
Declare.i 	GetProcAddress (handle, func$)
Declare 	FreeLibrary (handle)
Declare 	FreeLibraries()

EndDeclareModule

; IDE Options = PureBasic 5.60 (Windows - x86)
; CursorPosition = 38
; FirstLine = 3
; Folding = -
; EnableXP
; EnableUser
; CPU = 1
; CompileSourceDirectory
; EnablePurifier
; EnableExeConstant
; EnableUnicode